# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import AstmMessageV2, HeaderSegment
from driverng.astm import HprimDate as HL7Date
from driverng.astm import HprimForeign as HL7Foreign
from driverng.astm import HprimString as HL7String
from driverng.astm import Segment


class MassLynxSegmentMSH(HeaderSegment):
    sending_application = HL7String(3)
    sending_facility = HL7String(4)
    receiving_application = HL7String(5)
    receiving_facility = HL7String(6)
    date = HL7Date(7)
    security = HL7String(8)
    message_type = HL7String((9, 1), default="OML")
    message_code = HL7String((9, 2), default="O21")
    message_control_id = HL7String(10)
    processing_id = HL7String(11, default="P")
    version_id = HL7String(12, default="2.5")
    sequence_number = HL7String(13)
    character_set = HL7String(14)
    principal_language_message = HL7String(19)

    patients = HL7Foreign("PID", "header")


class MassLynxSegmentPID(Segment):
    patient_id = HL7String(3)
    patient_identifier_list = HL7String(4)
    patient_lastname = HL7String((6, 1))
    patient_firstname = HL7String((6, 2))
    mother_maiden_name = HL7String(7)
    birthdate = HL7Date(8, fmt_date="%Y%m%d")
    sex = HL7String(9)

    orders = HL7Foreign("OBR", "patient")
    info_action = HL7Foreign("ORC", "patient")

    def to_string(self, separators=None):
        self.rang_segment = ""
        return super(MassLynxSegmentPID, self).to_string(separators=separators)


class MassLynxSegmentORC(Segment):
    order_control = HL7String(1)

    orders = HL7Foreign("OBR", "patient")
    results = HL7Foreign("OBX", "orders")

    def to_string(self, separators=None):
        self.rang_segment = "NW"
        return super(MassLynxSegmentORC, self).to_string(separators=separators)


class MassLynxSegmentOBR(Segment):
    place_order_number = HL7String(2)
    filler_order_number = HL7String(3)
    universal_service_identifier = HL7String(4)
    assay_id = HL7String(5)
    requested_date = HL7Date(6)
    observation_date = HL7Date(8)
    # Injection volume
    collection_volume = HL7String(10)
    collection_identifier = HL7String(11)
    specimen_action_code = HL7String(12)
    danger_code = HL7String(13)
    relevant_clinical_information = HL7String(14)
    specimen_received_date = HL7Date(15)
    specimen_source = HL7String(16)
    ordering_provider = HL7String(17)
    order_callback_phone = HL7String(18)
    placer_field1 = HL7String(19)
    placer_field2 = HL7String(20)
    sample_type = HL7String(21)
    filler_field2 = HL7String(22)
    results_rpt_date = HL7Date(23)
    charge_to_practice = HL7String(24)
    diagnostic_serv = HL7String(25)
    result_status = HL7String(26)
    parent_result = HL7String(27)
    quantity = HL7String(28)
    result_copies_to = HL7String(29)
    parent = HL7String(30)
    transportation_mode = HL7String(31)
    reason_for_study = HL7String(32)
    principal_result_interpreter = HL7String(33)

    results = HL7Foreign("OBX", "orders")
    info_action = HL7Foreign("ORC", "orders")

    def to_string(self, separators=None):
        self.rang_segment = ""
        return super(MassLynxSegmentOBR, self).to_string(separators=separators)


class MassLynxSegmentOBX(Segment):
    value_type = HL7String(2)
    observation_identifier = HL7String((3, 1))
    observation_sub_id = HL7String((4, 1))
    compound_name = HL7String((4, 2))
    observation_value = HL7String(6)
    units = HL7String(7)
    references_range = HL7String(8)
    abnormal_flags = HL7String(9)
    probability = HL7String(10)
    nature_of_abnormal_test = HL7String(11)
    result_status = HL7String(12)
    effective_date_reference_range = HL7Date(13)
    user_defined_access_checks = HL7String(14)
    observation_date = HL7Date(15)
    producer_id = HL7String(16)
    responsible_observer = HL7String(17)
    observation_method = HL7String(18)
    equipment_identifier = HL7String(19)
    analysis_date = HL7Date(20)


class Driver(AstmMessageV2):
    SPEC = {"code": "masslynx", "label": "MassLynx HL7 WATERS"}
    SEGMENT_CLASS_MAP = {
        "MSH": MassLynxSegmentMSH,
        "PID": MassLynxSegmentPID,
        "ORC": MassLynxSegmentORC,
        "OBR": MassLynxSegmentOBR,
        "OBX": MassLynxSegmentOBX,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    def create(self, **kwargs):
        return MassLynxSegmentMSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    def encode_worklist(self, worklist, context):
        message = self.create()
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: None},
        ):
            for sample, action in patient.samples:
                seg_pid = message.patients.add(
                    patient_identifier_list=patient.patient_id,
                    patient_lastname=patient.last_name,
                    patient_firstname=patient.first_name,
                    mother_maiden_name=patient.maiden_name,
                    birthdate=patient.birthdate,
                    sex=patient.sex,
                )
                for order in sample["orders"]:
                    seg_orc = seg_pid.info_action.add()
                    seg_obr = seg_orc.orders.add(
                        filler_order_number=sample.sample_id,
                        assay_id=order.code,
                        observation_date=sample.request_date,
                        sample_type=sample.specimen,
                    )
                for result in sample.results:
                    seg_obr.collection_volume = result.value
        return message

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_pid in message.patients:
            for seg_obr in seg_pid.orders:
                sample = worklist.get_or_create(
                    sample_id=seg_obr.filler_order_number, type="patient"
                )
                if seg_obr.info_action:
                    results = seg_obr.info_action[0].results
                else:
                    results = seg_obr.results
                for seg_obx in results:
                    result = sample.results.add(
                        code=seg_obx.compound_name,
                        value=seg_obx.observation_value,
                        unit=seg_obx.units,
                        date=message.date,
                    )
                    if seg_obx.abnormal_flags:
                        result.flags = [seg_obx.abnormal_flags]

        return worklist
